@@ -1,13 +1,15 @@
+"""Classes for HDF5/MintPy file creation / writing."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
 # Author: Heresh Fattahi, Zhang Yunjun, 2017               #
 ############################################################
-# class used for data loading from InSAR stack to MintPy timeseries
 # Recommend import:
-#     from mintpy.objects.stackDict import (geometryDict,
-#                                           ifgramStackDict,
-#                                           ifgramDict)
+#   from mintpy.objects.stackDict import (
+#       geometryDict,
+#       ifgramStackDict,
+#       ifgramDict,
+#   )
 
 
 import os
@@ -749,7 +751,7 @@ class geometryDict:
                                           compression=compression)
 
             ###############################
-            # Generate Dataset if not existed in binary file: incidenceAngle, slantRangeDistance
+            # Generate Dataset if it doesn't exist as a binary file: incidenceAngle, slantRangeDistance
             for dsName in [i for i in ['incidenceAngle', 'slantRangeDistance'] if i not in self.dsNames]:
                 # Calculate data
                 data = None
