@@ -1,10 +1,11 @@
+"""Utilities for plotting."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
 # Author: Zhang Yunjun, 2018                               #
 ############################################################
 # Recommend import:
-#     from mintpy.utils import plot as pp
+#   from mintpy.utils import plot as pp
 
 
 import datetime as dt
@@ -125,9 +126,10 @@ def auto_figure_size(ds_shape, scale=1.0, disp_cbar=False, disp_slider=False,
                     max_figsize_height / fig_shape[1])
 
     # fig_shape/scale --> fig_size
-    fig_size = [i*fig_scale*scale for i in fig_shape]
+    fig_size = [x*fig_scale*scale for x in fig_shape]
+    fig_size = [float(f'{x:.1f}') for x in fig_size]
     if print_msg:
-        print(f'figure size : [{fig_size[0]:.2f}, {fig_size[1]:.2f}]')
+        print(f'figure size : [{fig_size[0]}, {fig_size[1]}]')
 
     return fig_size
 
@@ -456,7 +458,11 @@ def auto_adjust_xaxis_date(ax, datevector, fontsize=12, every_year=None, buffer_
 
     # auto param
     if not every_year:
-        every_year = max(1, np.rint((xmax - xmin).days / 365.25 / 5).astype(int))
+        # take axes width into account
+        fig = ax.get_figure()
+        bbox = ax.get_window_extent().transformed(fig.dpi_scale_trans.inverted())
+        scale = 6.2 / bbox.width
+        every_year = max(1, np.rint(scale * (xmax - xmin).days / 365.25 / 5).astype(int))
 
     if not every_month:
         if   every_year <= 3 :  every_month = 1
@@ -931,7 +937,7 @@ def plot_coherence_matrix(ax, date12List, cohList, date12List_drop=[], p_dict={}
     return ax, coh_mat, im
 
 
-def plot_num_triplet_with_nonzero_integer_ambiguity(fname, display=False, font_size=12, fig_size=[9,3]):
+def plot_num_triplet_with_nonzero_integer_ambiguity(fname, disp_fig=False, font_size=12, fig_size=[9,3]):
     """Plot the histogram for the number of triplets with non-zero integer ambiguity.
 
     Fig. 3d-e in Yunjun et al. (2019, CAGEO).
@@ -939,6 +945,10 @@ def plot_num_triplet_with_nonzero_integer_ambiguity(fname, display=False, font_s
     Parameters: fname - str, path to the numTriNonzeroIntAmbiguity.h5 file.
     """
 
+    # matplotlib backend setting
+    if not disp_fig:
+        plt.switch_backend('Agg')
+
     # read data
     data, atr = readfile.read(fname)
     vmax = int(np.nanmax(data))
@@ -983,7 +993,7 @@ def plot_num_triplet_with_nonzero_integer_ambiguity(fname, display=False, font_s
     print('plot and save figure to file', out_fig)
     fig.savefig(out_fig, bbox_inches='tight', transparent=True, dpi=300)
 
-    if display:
+    if disp_fig:
         plt.show()
     else:
         plt.close(fig)
@@ -1003,6 +1013,10 @@ def plot_timeseries_rms(rms_file, cutoff=3, out_fig=None, disp_fig=True,
                 fig_size - list of 2 float, figure size in inch
     """
 
+    # matplotlib backend setting
+    if not disp_fig:
+        plt.switch_backend('Agg')
+
     # read date / RMS info from text file
     fc = np.loadtxt(rms_file, dtype=bytes).astype(str)
     rms_list = fc[:, 1].astype(np.float32) * 1000.
@@ -1115,7 +1129,7 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
     site_names, site_lats, site_lons = gps.search_gps(SNWE, start_date, end_date)
     if site_names.size == 0:
         warnings.warn(f'No GNSS found within {SNWE} during {start_date} - {end_date}!')
-        print('Continue without GNSS plots.')
+        print('  continue without GNSS plots.')
         return ax
 
     # mask out stations not coincident with InSAR data
@@ -1140,6 +1154,7 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
         inps.gps_component = None
         vprint(f'WARNING: --gps-comp is not implemented for {k} file yet, set --gps-comp = None and continue')
 
+    plot_kwargs = dict(s=inps.gps_marker_size**2, edgecolors='k', lw=0.5, zorder=10)
     if inps.gps_component:
         # plot GPS velocity/displacement along LOS direction
         vprint('-'*30)
@@ -1201,28 +1216,12 @@ def plot_gps(ax, SNWE, inps, metadata=dict(), print_msg=True):
         for lat, lon, obs in zip(site_lats, site_lons, site_obs):
             if not np.isnan(obs):
                 color = cmap( (obs - vmin) / (vmax - vmin) )
-                ax.scatter(
-                    lon,
-                    lat,
-                    color=color,
-                    s=inps.gps_marker_size**2,
-                    edgecolors='k',
-                    lw=0.5,
-                    zorder=10,
-                )
+                ax.scatter(lon, lat, color=color, **plot_kwargs)
 
     else:
         # plot GPS locations only
         vprint('showing GPS locations')
-        ax.scatter(
-            site_lons,
-            site_lats,
-            s=inps.gps_marker_size**2,
-            color='w',
-            edgecolors='k',
-            lw=0.5,
-            zorder=10,
-        )
+        ax.scatter(site_lons, site_lats, color='w', **plot_kwargs)
 
     # plot GPS label
     if inps.disp_gps_label:
@@ -1415,6 +1414,8 @@ def plot_colorbar(inps, im, cax):
         else:
             cbar.locator = ticker.MaxNLocator(nbins=inps.cbar_nbins)
             cbar.update_ticks()
+    elif inps.cbar_ticks:
+        cbar.set_ticks(inps.cbar_ticks)
 
     cbar.ax.tick_params(which='both', direction='out', labelsize=inps.font_size, colors=inps.font_color)
 
@@ -1427,6 +1428,49 @@ def plot_colorbar(inps, im, cax):
     return inps, cbar
 
 
+def plot_faultline(ax, faultline_file, SNWE, linewidth=0.5, min_dist=0.1, print_msg=True):
+    """Plot fault lines.
+
+    Parameters: ax             - matplotlib.axes object
+                faultline_file - str, path to the fault line file in GMT lonlat format
+                SNWE           - tuple of 4 float, for south, north, west and east
+    Returns:    ax             - matplotlib.axes object
+                faults         - list of 2D np.ndarray in size of [num_point, 2] in float32
+                                 with each row for one point in [lon, lat] in degrees
+    """
+
+    if print_msg:
+        print(f'plot fault lines from GMT lonlat file: {faultline_file}')
+
+    # read faults
+    faults = readfile.read_gmt_lonlat_file(
+        faultline_file,
+        SNWE=SNWE,
+        min_dist=min_dist,
+        print_msg=print_msg,
+    )
+
+    if len(faults) == 0:
+        warnings.warn(f'No fault lines found within {SNWE} with length >= {min_dist} km!')
+        print('  continue without fault lines.')
+        return ax, faults
+
+    # plot
+    print_msg = False if len(faults) < 1000 else print_msg
+    prog_bar = ptime.progressBar(maxValue=len(faults), print_msg=print_msg)
+    for i, fault in enumerate(faults):
+        ax.plot(fault[:,0], fault[:,1], 'k-', lw=linewidth)
+        prog_bar.update(i+1, every=10)
+    prog_bar.close()
+
+    # keep the same axis limit
+    S, N, W, E = SNWE
+    ax.set_xlim(W, E)
+    ax.set_ylim(S, N)
+
+    return ax, faults
+
+
 def add_arrow(line, position=None, direction='right', size=15, color=None):
     """Add an arrow to a line.
 
@@ -1691,8 +1735,8 @@ def read_mask(fname, mask_file=None, datasetName=None, box=None, xstep=1, ystep=
             else:
                 mask_file = None
 
-    # Read mask file if inputed
-    mask = None
+    # read mask_data from file if inputed
+    mask_data = None
     if os.path.isfile(str(mask_file)):
         try:
             atr_msk = readfile.read_attribute(mask_file)
@@ -1704,7 +1748,7 @@ def read_mask(fname, mask_file=None, datasetName=None, box=None, xstep=1, ystep=
                     dsName = f'connectComponent-{date12}'
 
                 # read mask data
-                mask = readfile.read(
+                mask_data = readfile.read(
                     mask_file,
                     box=box,
                     datasetName=dsName,
@@ -1727,38 +1771,52 @@ def read_mask(fname, mask_file=None, datasetName=None, box=None, xstep=1, ystep=
     elif k in ['HDFEOS']:
         if datasetName.split('-')[0] in TIMESERIES_DSET_NAMES:
             mask_file = fname
-            mask = readfile.read(fname, datasetName='mask', print_msg=print_msg)[0]
+            mask_data = readfile.read(fname, datasetName='mask', print_msg=print_msg)[0]
             vprint(f'read {k} contained mask dataset.')
 
     elif fname.endswith('PARAMS.h5'):
         mask_file = fname
         with h5py.File(fname, 'r') as f:
-            mask = f['cmask'][:] == 1.
+            mask_data = f['cmask'][:] == 1.
         vprint(f'read {os.path.basename(fname)} contained cmask dataset')
 
     # multilook
-    if mask is not None and xstep * ystep > 1:
+    if mask_data is not None and xstep * ystep > 1:
         # output size if x/ystep > 1
-        xsize = int(mask.shape[1] / xstep)
-        ysize = int(mask.shape[0] / ystep)
+        xsize = int(mask_data.shape[1] / xstep)
+        ysize = int(mask_data.shape[0] / ystep)
 
         # sampling
-        mask = mask[int(ystep/2)::ystep,
-                    int(xstep/2)::xstep]
-        mask = mask[:ysize, :xsize]
+        mask_data = mask_data[int(ystep/2)::ystep,
+                              int(xstep/2)::xstep]
+        mask_data = mask_data[:ysize, :xsize]
 
-    # set to bool type
-    if mask is not None:
-        mask[np.isnan(mask)] = 0
+    # convert mask_data to mask (via thresholding, value translation, etc.)
+    if mask_data is None:
+        mask = None
+    else:
+        mask = np.array(mask_data)
 
-        # vmin/max
+        # vmin/vmax: create mask based on the input thresholds
         if vmin is not None:
-            mask[mask < vmin] = 0
+            mask = mask_data >= vmin
             vprint(f'hide pixels with mask value < {vmin}')
         if vmax is not None:
-            mask[mask > vmax] = 0
+            mask = mask_data <= vmax
             vprint(f'hide pixels with mask value > {vmax}')
 
+        # numTriNonzeroIntAmbiguity: keep pixels in 0, i.e. 0 -> 1, the rest -> 0
+        if os.path.basename(mask_file).startswith('numTriNonzeroIntAmbiguity'):
+            if vmin is None and vmax is None:
+                vprint('keep pixels with numTriNonzeroIntAmbiguity == 0 and mask out the rest')
+                mask = mask_data == 0
+            else:
+                vprint('--mask-vmin/vmax is specified, skip translating numTriNonzeroIntAmbiguity values')
+
+        # nan: mask out pixels in nan
+        mask[np.isnan(mask_data)] = 0
+
+        # ensure output in bool type
         mask = mask != 0
 
     return mask, mask_file
