@@ -1,3 +1,4 @@
+"""Utilities to read files."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
@@ -40,7 +41,7 @@ STD_METADATA_KEYS = {
     'AZIMUTH_PIXEL_SIZE' : ['azimuthPixelSize', 'azimuth_pixel_spacing', 'az_pixel_spacing', 'azimuth_spacing'],
     'RANGE_PIXEL_SIZE'   : ['rangePixelSize', 'range_pixel_spacing', 'rg_pixel_spacing', 'range_spacing'],
     'CENTER_LINE_UTC'    : ['center_time'],
-    'DATA_TYPE'          : ['dataType', 'data_type'],
+    'DATA_TYPE'          : ['dataType', 'data_type', 'image_format'],
     'EARTH_RADIUS'       : ['earthRadius', 'earth_radius_below_sensor', 'earth_radius'],
     'HEADING'            : ['HEADING_DEG', 'heading', 'centre_heading'],
     'HEIGHT'             : ['altitude', 'SC_height'],
@@ -119,6 +120,7 @@ DATA_TYPE_NUMPY2ENVI = {
 }
 
 # 2 - GDAL
+# link: https://gdal.org/api/raster_c_api.html#_CPPv412GDALDataType
 DATA_TYPE_GDAL2NUMPY = {
     1 : 'uint8',
     2 : 'uint16',
@@ -131,21 +133,25 @@ DATA_TYPE_GDAL2NUMPY = {
     9 : 'cint32',       # for translation purpose only, as numpy does not support complex int
     10: 'complex64',
     11: 'complex128',
+    12: 'uint64',
+    13: 'int64',
 }
 
 DATA_TYPE_NUMPY2GDAL = {
-    "uint8"     : 1,
-    "int8"      : 1,
-    "uint16"    : 2,
-    "int16"     : 3,
-    "uint32"    : 4,
-    "int32"     : 5,
-    "float32"   : 6,
-    "float64"   : 7,
-    "cint16"    : 8,    # for translation purpose only, as numpy does not support complex int
-    "cint32"    : 9,    # for translation purpose only, as numpy does not support complex int
-    "complex64" : 10,
-    "complex128": 11,
+    "uint8"     : 1,    # GDT_Byte
+    "int8"      : 1,    # GDT_Int8   (GDAL >= 3.7)
+    "uint16"    : 2,    # GDT_UInt16
+    "int16"     : 3,    # GDT_Int16
+    "uint32"    : 4,    # GDT_UInt32
+    "int32"     : 5,    # GDT_Int32
+    "float32"   : 6,    # GDT_Float32
+    "float64"   : 7,    # GDT_Float64
+    "cint16"    : 8,    # GDT_CInt16, for translation purpose only, as numpy does not support complex int
+    "cint32"    : 9,    # GDT_CInt32, for translation purpose only, as numpy does not support complex int
+    "complex64" : 10,   # GDT_CFloat32
+    "complex128": 11,   # GDT_CFloat64
+    "uint64"    : 12,   # GDT_UInt64 (GDAL >= 3.5)
+    "int64"     : 13,   # GDT_Int64  (GDAL >= 3.5)
 }
 
 # 3 - ISCE
@@ -167,9 +173,14 @@ DATA_TYPE_NUMPY2ISCE = {
     'complex64': 'CFLOAT',
 }
 
+# 4 - GAMMA
+DATA_TYPE_GAMMA2NUMPY = {
+    'fcomplex' : 'float64',
+}
 
 # single file (data + attributes) supported by GDAL
-GDAL_FILE_EXTS = ['.tiff', '.tif', '.grd']
+# .cos file - TerraSAR-X complex SAR data (https://gdal.org/drivers/raster/cosar.html)
+GDAL_FILE_EXTS = ['.tiff', '.tif', '.grd', '.cos']
 
 ENVI_BAND_INTERLEAVE = {
     'BAND' : 'BSQ',
@@ -511,19 +522,26 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
 
     # default data to read
     band = 1
-    cpx_band = 'phase'
+    if datasetName:
+        if datasetName.startswith(('mag', 'amp')):
+            cpx_band = 'magnitude'
+        elif datasetName in ['phase', 'angle']:
+            cpx_band = 'phase'
+        elif datasetName.lower() == 'real':
+            cpx_band = 'real'
+        elif datasetName.lower().startswith('imag'):
+            cpx_band = 'imag'
+        else:
+            cpx_band = 'complex'
+    else:
+        # use phase as default value, since it's the most common one.
+        cpx_band = 'phase'
 
     # ISCE
     if processor in ['isce']:
         # convert default short name for data type from ISCE
-        data_type_dict = {
-            'byte': 'int8',
-            'float': 'float32',
-            'double': 'float64',
-            'cfloat': 'complex64',
-        }
-        if data_type in data_type_dict.keys():
-            data_type = data_type_dict[data_type]
+        if data_type in DATA_TYPE_ISCE2NUMPY.keys():
+            data_type = DATA_TYPE_ISCE2NUMPY[data_type]
 
         ftype = atr['FILE_TYPE'].lower().replace('.', '')
         if ftype in ['unw', 'cor', 'ion']:
@@ -555,18 +573,6 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
                 band = 2
             elif datasetName.lower() == 'band3':
                 band = 3
-
-            elif datasetName.startswith(('mag', 'amp')):
-                cpx_band = 'magnitude'
-            elif datasetName in ['phase', 'angle']:
-                cpx_band = 'phase'
-            elif datasetName.lower() == 'real':
-                cpx_band = 'real'
-            elif datasetName.lower().startswith('imag'):
-                cpx_band = 'imag'
-            elif datasetName.startswith(('cpx', 'complex')):
-                cpx_band = 'complex'
-
             else:
                 # flexible band list
                 ds_list = get_slice_list(fname)
@@ -575,16 +581,15 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
 
         band = min(band, num_band)
 
+        # check file size
+        fsize = os.path.getsize(fname)
+        dsize = np.dtype(data_type).itemsize * length * width * num_band
+        if dsize != fsize:
+            warnings.warn(f'file size ({fsize}) does NOT match with metadata ({dsize})!')
+
     # ROI_PAC
     elif processor in ['roipac']:
-        # data structure - auto
-        interleave = 'BIL'
-        byte_order = 'little-endian'
-
         # data structure - file specific based on file extension
-        data_type = 'float32'
-        num_band = 1
-
         if fext in ['.unw', '.cor', '.hgt', '.msk']:
             num_band = 2
             band = 2
@@ -610,25 +615,27 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
     # Gamma
     elif processor == 'gamma':
         # data structure - auto
-        interleave = 'BIL'
         byte_order = atr.get('BYTE_ORDER', 'big-endian')
 
-        data_type = 'float32'
+        # convert default short name for data type from GAMMA
+        if data_type in DATA_TYPE_GAMMA2NUMPY.keys():
+            data_type = DATA_TYPE_GAMMA2NUMPY[data_type]
+
         if fext in ['.unw', '.cor', '.hgt_sim', '.dem', '.amp', '.ramp']:
             pass
 
         elif fext in ['.int']:
-            data_type = 'complex64'
+            data_type = data_type if 'DATA_TYPE' in atr.keys() else 'complex64'
 
-        elif fext in ['.utm_to_rdc']:
-            data_type = 'float32'
+        elif fext.endswith(('to_rdc', '2_rdc', '2rdc')):
+            data_type = data_type if 'DATA_TYPE' in atr.keys() else 'float32'
             interleave = 'BIP'
             num_band = 2
             if datasetName and datasetName.startswith(('az', 'azimuth')):
                 band = 2
 
         elif fext == '.slc':
-            data_type = 'complex32'
+            data_type = data_type if 'DATA_TYPE' in atr.keys() else 'complex32'
             cpx_band = 'magnitude'
 
         elif fext in ['.mli']:
@@ -694,14 +701,7 @@ def get_slice_list(fname, no_complex=False):
     Returns:    slice_list - list(str), list of names for 2D matrices
     """
 
-    # grab fbase/fext
-    fbase, fext = os.path.splitext(os.path.basename(fname))
-    fext = fext.lower()
-    # ignore certain meaningless file extensions
-    while fext in ['.geo', '.rdr', '.full', '.wgs84', '.grd']:
-        fbase, fext = os.path.splitext(fbase)
-    fext = fext if fext else fbase
-
+    fbase, fext = _get_file_base_and_ext(fname)
     atr = read_attribute(fname)
     ftype = atr['FILE_TYPE']
 
@@ -767,12 +767,14 @@ def get_slice_list(fname, no_complex=False):
 
             # special order for velocity / time func file
             if ftype == 'velocity':
-                slice_list = sort_dataset_list4velocity(slice_list)
+                slice_list = _sort_dataset_list4velocity(slice_list)
 
     # Binary Files
     else:
         num_band = int(atr.get('BANDS', '1'))
-        if fext in ['.trans', '.utm_to_rdc']:
+        dtype = atr.get('DATA_TYPE', 'float32')
+
+        if fext in ['.trans'] or fext.endswith(('to_rdc', '2_rdc', '2rdc')):
             # roipac / gamma lookup table
             slice_list = ['rangeCoord', 'azimuthCoord']
 
@@ -783,18 +785,18 @@ def get_slice_list(fname, no_complex=False):
         elif fext in ['.unw', '.ion']:
             slice_list = ['magnitude', 'phase']
 
-        elif fext in ['.int', '.slc']:
+        elif fext in ['.int', '.slc'] or (dtype.startswith('c') and num_band == 1):
             if no_complex:
                 slice_list = ['magnitude', 'phase']
             else:
                 slice_list = ['complex']
 
-        elif fbase.startswith('off') and fext in ['.bip'] and num_band == 2:
-            # ampcor offset file
+        elif 'offset' in fbase and num_band == 2:
+            # ampcor offset file, e.g. offset.bip, dense_offsets.bil
             slice_list = ['azimuthOffset', 'rangeOffset']
 
-        elif fbase.startswith('off') and fname.endswith('cov.bip') and num_band == 3:
-            # ampcor offset covariance file
+        elif 'offset' in fbase and '_cov' in fbase and num_band == 3:
+            # ampcor offset covariance file, e.g. offset_cov.bip, dense_offsets_cov.bil
             slice_list = ['azimuthOffsetVar', 'rangeOffsetVar', 'offsetCovar']
 
         elif fext in ['.lkv']:
@@ -838,7 +840,7 @@ def get_dataset_list(fname, datasetName=None):
 
         # special order for velocity / time func file
         if atr['FILE_TYPE'] == 'velocity':
-            ds_list = sort_dataset_list4velocity(ds_list)
+            ds_list = _sort_dataset_list4velocity(ds_list)
 
     else:
         ds_list = get_slice_list(fname)
@@ -846,7 +848,7 @@ def get_dataset_list(fname, datasetName=None):
     return ds_list
 
 
-def sort_dataset_list4velocity(ds_list_in):
+def _sort_dataset_list4velocity(ds_list_in):
     """Sort the dataset list for velocity file type.
 
     1. time func datasets [required]: velocity
@@ -912,7 +914,11 @@ def get_hdf5_compression(fname):
 
 
 def get_no_data_value(fname):
-    """Grab the NO_DATA_VALUE of the input file."""
+    """Grab the NO_DATA_VALUE of the input file.
+
+    Parameters: fname - str, path to the data file
+    Returns:    val   - number, no data value
+    """
     val = read_attribute(fname).get('NO_DATA_VALUE', None)
     val = str(val).lower()
 
@@ -925,6 +931,26 @@ def get_no_data_value(fname):
     return val
 
 
+def _get_file_base_and_ext(fname):
+    """Grab the meaningful file basename and extension.
+
+    Parameters: fname - str, path to the (meta)data file
+    Returns:    fbase - str, file basename
+                fext  - str, file extension in lower case
+    """
+
+    fbase, fext = os.path.splitext(os.path.basename(fname))
+    fext = fext.lower()
+
+    # ignore certain meaningless file extensions
+    while fext in ['.geo', '.rdr', '.full', '.mli', '.wgs84', '.grd', '.bil', '.bip']:
+        fbase, fext = os.path.splitext(fbase)
+    # set fext to fbase if nothing left
+    fext = fext if fext else fbase
+
+    return fbase, fext
+
+
 #########################################################################
 def read_attribute(fname, datasetName=None, metafile_ext=None):
     """Read attributes of input file into a dictionary
@@ -1181,10 +1207,7 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
         meta_ext = os.path.splitext(metafile)[1].lower()
 
         # ignore certain meaningless file extensions
-        while fext in ['.geo', '.rdr', '.full', '.wgs84', '.grd']:
-            fbase, fext = os.path.splitext(fbase)
-        if not fext:
-            fext = fbase
+        fbase, fext = _get_file_base_and_ext(fname)
 
         if meta_ext == '.rsc':
             atr.update(read_roipac_rsc(metafile))
@@ -1224,12 +1247,15 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
             atr.update(read_gdal_vrt(metafile))
             atr['FILE_TYPE'] = fext
 
-        # DATA_TYPE for ISCE products
+        # DATA_TYPE for ISCE/ROI_PAC products
         data_type_dict = {
-            'byte': 'int8',
-            'float': 'float32',
+            # isce2
+            'byte'  : 'int8',
+            'float' : 'float32',
             'double': 'float64',
             'cfloat': 'complex64',
+            # roipac
+            'ci2'   : 'float32',
         }
         data_type = atr.get('DATA_TYPE', 'none').lower()
         if data_type != 'none' and data_type in data_type_dict.keys():
@@ -1240,6 +1266,8 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
         # ignore Std because it shares the same unit as base parameter
         # e.g. velocityStd and velocity
         datasetName = datasetName.replace('Std','')
+        # use the last segment if / is used, e.g. HDF-EOS5
+        datasetName = datasetName.split('/')[-1]
     ftype = atr['FILE_TYPE'].replace('.', '')
     if ftype == 'ifgramStack':
         if datasetName and datasetName in DSET_UNIT_DICT.keys():
@@ -1259,41 +1287,68 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
         else:
             atr['UNIT'] = '1'
 
-    # NO_DATA_VALUE
-    no_data_value = atr.get('NO_DATA_VALUE', None)
-    atr['NO_DATA_VALUE'] = str(no_data_value).lower()
-
     # FILE_PATH
     if 'FILE_PATH' in atr.keys() and 'OG_FILE_PATH' not in atr.keys():
         # need to check original source file to successfully subset legacy-sensor products
         atr['OG_FILE_PATH'] = atr['FILE_PATH']
     atr['FILE_PATH'] = os.path.abspath(fname)
 
+    # NO_DATA_VALUE
+    atr['NO_DATA_VALUE'] = auto_no_data_value(atr)
+
     atr = standardize_metadata(atr)
 
     return atr
 
 
-def standardize_metadata(metaDictIn, standardKeys=STD_METADATA_KEYS):
-    """Convert metadata input ROI_PAC/MintPy format (for metadata with the same values)."""
+def auto_no_data_value(meta):
+    """Get default no-data-value for the given file's metadata.
+
+    Parameters: meta          - dict, metadata
+    Returns:    no_data_value - str, no data value in lower case string
+    """
+
+    if 'NO_DATA_VALUE' in meta.keys():
+        no_data_value = meta['NO_DATA_VALUE']
+
+    else:
+        processor = meta['PROCESSOR']
+        fname = meta['FILE_PATH']
+        fbase, fext = _get_file_base_and_ext(fname)
+        num_band = int(meta.get('BANDS', 0))
+
+        # known file types
+        # isce2: dense offsets from topsApp.py
+        if processor == 'isce' and fname.endswith('dense_offsets.bil') and num_band == 2:
+            no_data_value = -10000.
+
+        else:
+            # default value for unknown file types
+            no_data_value = None
+
+    return str(no_data_value).lower()
+
+
+def standardize_metadata(in_meta, standard_keys=STD_METADATA_KEYS):
+    """Convert metadata into ROI_PAC/MintPy format (for metadata with the same values)."""
 
     # make a copy
-    metaDict = dict()
-    for key, value in iter(metaDictIn.items()):
-        metaDict[key] = value
+    out_meta = dict()
+    for key, value in iter(in_meta.items()):
+        out_meta[key] = value
 
     # get potential keys to match
-    in_keys = [i for i in metaDict.keys() if i not in standardKeys.keys()]
-    std_keys = [i for i in standardKeys.keys() if i not in metaDict.keys()]
+    in_keys = [i for i in out_meta.keys() if i not in standard_keys.keys()]
+    std_keys = [i for i in standard_keys.keys() if i not in out_meta.keys()]
 
     # loop to find match and assign values
     for std_key in std_keys:
-        cand_keys = standardKeys[std_key]
+        cand_keys = standard_keys[std_key]
         cand_keys = [i for i in cand_keys if i in in_keys]
         if len(cand_keys) > 0:
-            metaDict[std_key] = metaDict[cand_keys[0]]
+            out_meta[std_key] = out_meta[cand_keys[0]]
 
-    return metaDict
+    return out_meta
 
 
 #########################################################################
@@ -1396,13 +1451,13 @@ def read_gamma_par(fname, delimiter=':', skiprows=3):
             value = str.replace(c[1], '\n', '').split("#")[0].split()[0].strip()
             parDict[key] = value
 
-    parDict = attribute_gamma2roipac(parDict)
+    parDict = _attribute_gamma2roipac(parDict)
     parDict = standardize_metadata(parDict)
 
     return parDict
 
 
-def attribute_gamma2roipac(par_dict_in):
+def _attribute_gamma2roipac(par_dict_in):
     """Convert Gamma metadata into ROI_PAC/MintPy format."""
     par_dict = dict()
     for key, value in iter(par_dict_in.items()):
@@ -1510,6 +1565,13 @@ def read_isce_xml(fname):
         if meta is not None:
             xmlDict['NoDataValue'] = meta.text
 
+    # configeration file, e.g. topsApp.xml
+    elif root.tag.endswith('App'):
+        for child in root[0].findall('property'):
+            key = child.get('name').lower()
+            value = child.find('value')
+            xmlDict[key] = value.text if value is not None else child.text
+
     # standardize metadata keys
     xmlDict = standardize_metadata(xmlDict)
 
@@ -1522,6 +1584,8 @@ def read_envi_hdr(fname):
     atr['DATA_TYPE'] = DATA_TYPE_ENVI2NUMPY[atr.get('data type', '4')]
     atr['BYTE_ORDER'] = ENVI_BYTE_ORDER[atr.get('byte order', '1')]
 
+    # ENVI seems to use the center of the upper-left pixel as the first coordinates
+    # link: https://www.l3harrisgeospatial.com/docs/OverviewMapInformationInENVI.html
     if 'map info' in atr.keys():
         map_info = [i.replace('{','').replace('}','').strip() for i in atr['map info'].split(',')]
         x_step = abs(float(map_info[5]))
@@ -1569,7 +1633,10 @@ def read_gdal_vrt(fname):
     atr['INTERLEAVE'] = ENVI_BAND_INTERLEAVE[interleave]
 
     # transformation contains gridcorners
-    # (lines/pixels or lonlat and the spacing 1/-1 or deltalon/deltalat)
+    #   lines/pixels with a spacing of 1/-1 OR lonlat with a spacing of deltalon/deltalat
+    # GDAL uses the upper-left corner of the upper-left pixel as the first coordinate,
+    #   which is the same as ROI_PAC and MintPy
+    #   link: https://gdal.org/tutorials/geotransforms_tut.html
     transform = ds.GetGeoTransform()
     x0 = transform[0]
     y0 = transform[3]
@@ -1578,8 +1645,8 @@ def read_gdal_vrt(fname):
 
     atr['X_STEP'] = x_step
     atr['Y_STEP'] = y_step
-    atr['X_FIRST'] = x0 - x_step / 2.
-    atr['Y_FIRST'] = y0 - y_step / 2.
+    atr['X_FIRST'] = x0
+    atr['Y_FIRST'] = y0
 
     # projection / coordinate unit
     srs = osr.SpatialReference(wkt=ds.GetProjection())
@@ -1645,13 +1712,13 @@ def read_gmtsar_prm(fname, delimiter='='):
         value = c[1].replace('\n', '').strip()
         prmDict[key] = value
 
-    prmDict = attribute_gmtsar2roipac(prmDict)
+    prmDict = _attribute_gmtsar2roipac(prmDict)
     prmDict = standardize_metadata(prmDict)
 
     return prmDict
 
 
-def attribute_gmtsar2roipac(prm_dict_in):
+def _attribute_gmtsar2roipac(prm_dict_in):
     """Convert GMTSAR metadata into ROI_PAC/MintPy format (for metadata with different values)."""
     prm_dict = dict()
     for key, value in iter(prm_dict_in.items()):
@@ -1843,36 +1910,51 @@ def read_binary(fname, shape, box=None, data_type='float32', byte_order='l',
     # read data
     interleave = interleave.upper()
     if interleave == 'BIL':
-        data = np.fromfile(fname,
-                           dtype=data_type,
-                           count=box[3]*width*num_band).reshape(-1, width*num_band)
-        data = data[box[1]:box[3],
-                    width*(band-1)+box[0]:width*(band-1)+box[2]]
+        count = box[3] * width * num_band
+        data = np.fromfile(fname, dtype=data_type, count=count).reshape(-1, width*num_band)
+        c0 = width * (band - 1) + box[0]
+        c1 = width * (band - 1) + box[2]
+        data = data[box[1]:box[3], c0:c1]
 
     elif interleave == 'BIP':
-        data = np.fromfile(fname,
-                           dtype=data_type,
-                           count=box[3]*width*num_band).reshape(-1, width*num_band)
-        data = data[box[1]:box[3],
-                    np.arange(box[0], box[2])*num_band+band-1]
+        count = box[3] * width * num_band
+        data = np.fromfile(fname, dtype=data_type, count=count).reshape(-1, width*num_band)
+        inds = np.arange(box[0], box[2]) * num_band + band - 1
+        data = data[box[1]:box[3], inds]
 
     elif interleave == 'BSQ':
-        data = np.fromfile(fname,
-                           dtype=data_type,
-                           count=(box[3]+length*(band-1))*width).reshape(-1, width)
-        data = data[length*(band-1)+box[1]:length*(band-1)+box[3],
-                    box[0]:box[2]]
+        count = (box[3] + length * (band - 1)) * width
+        data = np.fromfile(fname, dtype=data_type, count=count).reshape(-1, width)
+        r0 = length * (band - 1) + box[1]
+        r1 = length * (band - 1) + box[3]
+        data = data[r0:r1, box[0]:box[2]]
+
     else:
         raise ValueError('unrecognized band interleaving:', interleave)
 
     # adjust output band for complex data
     if data_type.replace('>', '').startswith('c'):
-        if   cpx_band.startswith('real'):  data = data.real
-        elif cpx_band.startswith('imag'):  data = data.imag
-        elif cpx_band.startswith('pha'):   data = np.angle(data)
-        elif cpx_band.startswith('mag'):   data = np.absolute(data)
-        elif cpx_band.startswith('c'):     pass
-        else:  raise ValueError('unrecognized complex band:', cpx_band)
+        if cpx_band.startswith('real'):
+            data = data.real
+        elif cpx_band.startswith('imag'):
+            data = data.imag
+
+        elif cpx_band.startswith('pha'):
+            data = np.angle(data)
+
+            # set ~pi value to 0, as sometimes shown in gamma products
+            boundary_values = [-3.1415927, 3.1415927]
+            for boundary_value in boundary_values:
+                if np.sum(data == boundary_value) > 100:
+                    print(f'~pi boundary value ({boundary_value}) detected, convert to zero')
+                    data[data == boundary_value] = 0
+
+        elif cpx_band.startswith(('mag', 'amp')):
+            data = np.absolute(data)
+        elif cpx_band.startswith(('cpx', 'complex')):
+            pass
+        else:
+            raise ValueError('unrecognized complex band:', cpx_band)
 
     # skipping/multilooking
     if xstep * ystep > 1:
@@ -1930,8 +2012,8 @@ def read_gdal(fname, box=None, band=1, cpx_band='phase', xstep=1, ystep=1):
             data = data.imag
         elif cpx_band.startswith('pha'):
             data = np.angle(data)
-        elif cpx_band.startswith('mag'):
-            data = np.absolute(data)
+        elif cpx_band.startswith(('mag', 'amp')):
+            data = np.abs(data)
         elif cpx_band.startswith(('cpx', 'complex')):
             pass
         else:
@@ -1954,7 +2036,7 @@ def read_gdal(fname, box=None, band=1, cpx_band='phase', xstep=1, ystep=1):
 
 ############################  Read GMT Faults  ################################
 
-def read_gmt_lonlat_file(ll_file, SNWE=None, min_dist=10):
+def read_gmt_lonlat_file(ll_file, SNWE=None, min_dist=10, print_msg=True):
     """Read GMT lonlat file into list of 2D np.ndarray.
 
     Parameters: ll_file  - str, path to the GMT lonlat file
@@ -1996,11 +2078,13 @@ def read_gmt_lonlat_file(ll_file, SNWE=None, min_dist=10):
         lines = lines[:1000]
 
     # loop to extract/organize the data into list of arrays
+    faults, fault = [], []
     num_line = len(lines)
-    faults = []
-    fault = []
-    prog_bar = ptime.progressBar(maxValue=num_line)
+    print_msg = False if num_line < 5000 else print_msg
+    prog_bar = ptime.progressBar(maxValue=num_line, print_msg=print_msg)
     for i, line in enumerate(lines):
+        prog_bar.update(i+1, every=1000, suffix=f'line {i+1} / {num_line}')
+
         line = line.strip().replace('\n','').replace('\t', ' ')
         if line.startswith('>'):
             fault = []
@@ -2027,9 +2111,8 @@ def read_gmt_lonlat_file(ll_file, SNWE=None, min_dist=10):
 
             if fault is not None:
                 faults.append(fault)
-
-        prog_bar.update(i+1, every=1000, suffix=f'line {i+1} / {num_line}')
     prog_bar.close()
+
     return faults
 
 
