@@ -1,3 +1,4 @@
+"""Miscellaneous utilities - dependent on utils0/1."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
@@ -303,7 +304,7 @@ def transect_yx(z, atr, start_yx, end_yx, interpolation='nearest'):
     # for nearest neighbor sampling, use indexing directly
     # for other interpolation, use scipy.ndimage.map_coordinates
     if interpolation == 'nearest':
-        z_line = z[np.rint(ys).astype(np.int), np.rint(xs).astype(np.int)]
+        z_line = z[np.rint(ys).astype(int), np.rint(xs).astype(int)]
 
     else:
         # interpolation name to order
@@ -464,8 +465,8 @@ def prepare_geo_los_geometry(geom_file, unit='rad'):
         E = W + x_step * width
 
         # SNWE in meter --> degree
-        lat0, lon0 = to_latlon(atr['OG_FILE_PATH'], W, N)
-        lat1, lon1 = to_latlon(atr['OG_FILE_PATH'], E, S)
+        lat0, lon0 = utm2latlon(atr, W, N)
+        lat1, lon1 = utm2latlon(atr, E, S)
         lat_step = (lat1 - lat0) / length
         lon_step = (lon1 - lon0) / width
 
