@@ -1,9 +1,9 @@
+"""Miscellaneous utilities - independent."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
 # Author: Zhang Yunjun, Heresh Fattahi, 2013               #
 ############################################################
-# Low level utilities script (independent)
 # Contents
 #   InSAR
 #   File Operation
@@ -24,9 +24,9 @@ import h5py
 import numpy as np
 
 # global variables
-SPEED_OF_LIGHT = 299792458 # m/s
-EARTH_RADIUS = 6371e3      # Earth radius in meters
-K = 40.31                  # m^3/s^2, constant
+SPEED_OF_LIGHT = 299792458  # m/s
+EARTH_RADIUS = 6371.0088e3  # Earth radius in meters
+K = 40.31                   # m^3/s^2, constant
 
 
 #################################### InSAR ##########################################
@@ -125,7 +125,7 @@ def incidence_angle(atr, dem=None, dimension=2, print_msg=True):
     # Read Attributes
     range_n = float(atr['STARTING_RANGE'])
     dR = float(atr['RANGE_PIXEL_SIZE'])
-    r = float(atr['EARTH_RADIUS'])
+    r = float(atr.get('EARTH_RADIUS', EARTH_RADIUS))
     H = float(atr['HEIGHT'])
     width = int(atr['WIDTH'])
 
@@ -228,47 +228,6 @@ def azimuth_ground_resolution(atr):
     return az_step
 
 
-def auto_lat_lon_step_size(atr, lat_c=None):
-    """Get the default lat/lon step size for geocoding.
-
-    Treat the pixel in radar coordinates as an rotated rectangle. Use the bounding box
-    of the rotated rectangle for the ratio between lat and lon steps. Then scale the
-    lat and lon step size to ensure the same area between the pixels in radar and geo
-    coordinates.
-
-    Link: https://math.stackexchange.com/questions/4001034
-
-    Parameters: atr      - dict, standard mintpy metadata
-                lat_c    - float, central latitude in degree
-    Returns:    lat_step - float, latitude  step size in degree
-                lon_step - float, longitude step size in degree
-    """
-    # azimuth angle (rotation angle) in radian
-    az_angle = np.deg2rad(abs(heading2azimuth_angle(float(atr['HEADING']))))
-
-    # radar pixel size in meter
-    az_step = azimuth_ground_resolution(atr)
-    rg_step = range_ground_resolution(atr)
-
-    # geo pixel size in meter
-    x_step = rg_step * abs(np.cos(az_angle)) + az_step * abs(np.sin(az_angle))
-    y_step = rg_step * abs(np.sin(az_angle)) + az_step * abs(np.cos(az_angle))
-    scale_factor = np.sqrt((rg_step * az_step) / (x_step * y_step))
-    x_step *= scale_factor
-    y_step *= scale_factor
-
-    # geo pixel size in degree
-    if lat_c is None:
-        if 'LAT_REF1' in atr.keys():
-            lat_c = (float(atr['LAT_REF1']) + float(atr['LAT_REF3'])) / 2.
-        else:
-            lat_c = 0
-    lon_step = np.rad2deg(x_step / (EARTH_RADIUS * np.cos(np.deg2rad(lat_c))))
-    lat_step = np.rad2deg(y_step / EARTH_RADIUS) * -1.
-
-    return lat_step, lon_step
-
-
 
 #################################### File Operation ##########################################
 def touch(fname_list, times=None):
@@ -300,21 +259,24 @@ def touch(fname_list, times=None):
 ################################## Coordinate ##########################################
 def utm_zone2epsg_code(utm_zone):
     """Convert UTM Zone string to EPSG code.
-    Parameters: utm_zone - str, atr['UTM_ZONE']
-    Returns:    epsg     - str, EPSG code
-    Examples:   epsg = utm_zone2epsg_code('11N')
+
+    Parameters: utm_zone  - str, atr['UTM_ZONE'], comprises a zone number
+                            and a hemisphere, e.g. 11N, 36S, etc.
+    Returns:    epsg_code - str, EPSG code
+    Examples:   epsg_code = utm_zone2epsg_code('11N')
     """
     from pyproj import CRS
-    crs = CRS.from_dict({'proj': 'utm',
-                         'zone': int(utm_zone[:-1]),
-                         'south': utm_zone[-1] == 'S',
-                        })
-    epsg = crs.to_authority()[1]
-    return epsg
+    crs = CRS.from_dict({
+        'proj': 'utm',
+        'zone': int(utm_zone[:-1]),
+        'south': utm_zone[-1].upper() == 'S',
+    })
+    epsg_code = crs.to_authority()[1]
+    return epsg_code
 
 
 def to_latlon(infile, x, y):
-    """Convert x, y in the projection coordinates of the file to lon/lat in degree.
+    """Convert x, y in the projection coordinates of the file to lat/lon in degree.
 
     Similar functionality also exists in utm.to_latlon() at:
         https://github.com/Turbo87/utm#utm-to-latitudelongitude
@@ -343,6 +305,35 @@ def to_latlon(infile, x, y):
     return y, x
 
 
+def utm2latlon(meta, easting, northing):
+    """Convert UTM easting/northing in meters to lat/lon in degrees.
+
+    Parameters: meta     - dict, mintpy attributes that includes:
+                           UTM_ZONE
+                easting  - scalar or 1/2D np.ndarray, UTM    coordiantes in x direction
+                northing - scalar or 1/2D np.ndarray, UTM    coordiantes in y direction
+    Returns:    lat      - scalar or 1/2D np.ndarray, WGS 84 coordiantes in y direction
+                lon      - scalar or 1/2D np.ndarray, WGS 84 coordiantes in x direction
+    """
+    import utm
+    zone_num = int(meta['UTM_ZONE'][:-1])
+    northern = meta['UTM_ZONE'][-1].upper() == 'N'
+    lat, lon = utm.to_latlon(easting, northing, zone_num, northern=northern)
+    return lat, lon
+
+
+def latlon2utm(lat, lon):
+    """Convert latitude/longitude in degrees to UTM easting/northing in meters.
+
+    Parameters: lat      - scalar or 1/2D np.ndarray, WGS 84 coordiantes in y direction
+                lon      - scalar or 1/2D np.ndarray, WGS 84 coordiantes in x direction
+    Returns:    easting  - scalar or 1/2D np.ndarray, UTM    coordiantes in x direction
+                northing - scalar or 1/2D np.ndarray, UTM    coordiantes in y direction
+    """
+    import utm
+    return utm.from_latlon(lat, lon)[:2]
+
+
 def snwe_to_wkt_polygon(snwe):
     """Convert the input bounding box in SNWE into WKT format POLYGON.
 
@@ -410,6 +401,11 @@ def get_lat_lon(meta, geom_file=None, box=None, dimension=2, ystep=1, xstep=1):
         else:
             raise ValueError(f'un-supported dimension = {dimension}')
 
+        # UTM to lat/lon
+        if not meta['Y_UNIT'].startswith('deg') and 'UTM_ZONE' in meta.keys():
+            print('UTM coordinates detected, convert UTM into lat/lon')
+            lats, lons = utm2latlon(meta, easting=lons, northing=lats)
+
     else:
         msg = 'Can not get pixel-wise lat/lon!'
         msg += '\nmeta dict is not geocoded and/or geometry file does not contains latitude/longitude dataset.'
@@ -663,7 +659,7 @@ def en2az(v_e, v_n, orb_az_angle):
                 v_n          - np.ndarray or float, displacement in north-south direction, north as positive
                 orb_az_angle - np.ndarray or float, azimuth angle of the SAR platform along track/orbit direction
                                measured from the north with anti-clockwise direction as positive, in the unit of degrees
-                               orb_az_angle = los_az_angle + 90 for right-looking radar.
+                               orb_az_angle = los_az_angle - 90 for right-looking radar.
     Returns:    v_az         - np.ndarray or float, displacement in azimuth direction,
                                motion along flight direction as positive
     """
@@ -1073,41 +1069,47 @@ def print_command_line(script_name, args):
 
 #################################### Math / Statistics ###################################
 def median_abs_deviation(data, center=None, scale=0.67449):
-    """Compute the median absolute deviation of the data along the LAST axis.
+    """Compute the median absolute deviation (MAD) of the data along the LAST axis.
 
-    This function is also included as:
-        scipy.stats.median_abs_deviation() in scipy v1.5.0
-            https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.median_abs_deviation.html
-        statsmodels.robust.mad() in statsmodels
-            https://www.statsmodels.org/dev/generated/statsmodels.robust.scale.mad.html
+    This function is also available in the following packages:
+    + statsmodels.robust.mad()
+      https://www.statsmodels.org/dev/generated/statsmodels.robust.scale.mad.html
+    + scipy.stats.median_abs_deviation() since v1.5.0
+      https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.median_abs_deviation.html
 
-    The differences are:
-        1. scipy: out = out / scale while mintpy: out = out * scale
-        2. scipy propagates nan by default, while mintpy omit nan by default.
+    The implementation here is preferred because we would like to:
+    1. omit the NaN value in the data for the median and MAD calculation
+    2. scale the returned value to be comparable with standard deviation (STD)
+       for easy interpretation with 1/2/3-sigma rule.
+    While statsmodels could not handle the NaN value in the data; and scipy does not hebave
+    as desired in the default setting (propagate nan and do not scale).
 
     The following two are equivalent for a 2D np.ndarray X:
-        mintpy.utils.utils0.median_abs_deviation(X)
-        scipy.stats.median_abs_deviation(X, axis=-1, center=np.nanmedian, scale=1./0.67449, nan_policy='omit')
+    + mintpy.utils.utils0.median_abs_deviation(X)
+    + scipy.stats.median_abs_deviation(X, axis=-1, center=np.nanmedian, scale=0.67449, nan_policy='omit')
 
     Parameters: data   - 1/2D np.ndarray, input array
                 center - 0/1D np.ndarray or None
                 scale  - float, the normalization constant
     Returns:    mad    - 0/1D np.ndarray
     """
-    # compute MAD over 1/2D matrix only
+    # check input: compute MAD over 1/2D matrix only
     data = np.array(data)
     if data.ndim > 2:
         ntime = data.shape[0]
         data = data.reshape(ntime, -1)
 
-    # default center value: median
+    # calculate the center value: median along the last axis
     if center is None:
         center = np.nanmedian(data, axis=-1)
 
-    # calculation
-    if data.ndim == 2:
-        center = np.tile(center.reshape(-1,1), (1, data.shape[1]))
-    mad = np.nanmedian(np.abs(data - center), axis=-1) * scale
+        # replicate center matrix for 2D matrix calculation later on
+        if data.ndim == 2:
+            center = np.tile(center.reshape(-1, 1), (1, data.shape[1]))
+
+    # calculate median absolute deviation
+    mad = np.nanmedian(np.abs(data - center), axis=-1) / scale
+
     return mad
 
 
