@@ -1,3 +1,4 @@
+"""Utilities to write files."""
 ############################################################
 # Program is part of MintPy                                #
 # Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
@@ -170,7 +171,8 @@ def write(datasetDict, out_file, metadata=None, ref_file=None, compression=None,
                 data_list = [data_list[0], data_list[0]]
                 meta['BANDS'] = 2
 
-        elif fext in ['.cor', '.hgt']:
+        elif fext in ['.cor']:
+            # remove .hgt as it can be float64 in isce2.
             meta['DATA_TYPE'] = 'float32'
             meta['INTERLEAVE'] = 'BIL'
             if meta.get('PROCESSOR', 'isce') == 'roipac':
@@ -184,7 +186,7 @@ def write(datasetDict, out_file, metadata=None, ref_file=None, compression=None,
             meta['DATA_TYPE'] = 'float32'
             meta['INTERLEAVE'] = 'BIL'
 
-        elif fext in ['.utm_to_rdc']:
+        elif fext.endswith(('to_rdc', '2_rdc', '2rdc')):
             # Gamma lookup table
             meta['BANDS'] = 2
             meta['DATA_TYPE'] = 'float32'
